Package: ggspavis
Version: 1.19.0
Title: Visualization functions for spatial transcriptomics data
Description: Visualization functions for spatial transcriptomics data. Includes 
    functions to generate several types of plots, including spot plots, feature 
    (molecule) plots, reduced dimension plots, spot-level quality control (QC) 
    plots, and feature-level QC plots, for datasets from the 10x Genomics 
    Visium and other technological platforms. Datasets are assumed to be in 
    either SpatialExperiment or SingleCellExperiment format.
Authors@R: c(
    person("Lukas M.", "Weber", 
        email = "lmweb012@gmail.com", 
        role = c("aut", "cre"), 
        comment = c(ORCID = "0000-0002-3282-1730")), 
    person("Helena L.", "Crowell", 
        email = "helena.crowell@uzh.ch", 
        role = c("aut"), 
        comment = c(ORCID = "0000-0002-4801-1767")), 
    person("Yixing E.", "Dong", 
        email = "estelladong729@gmail.com", 
        role = c("aut"), 
        comment = c(ORCID = "0009-0003-5115-5686")))
URL: https://github.com/lmweber/ggspavis
BugReports: https://github.com/lmweber/ggspavis/issues
License: MIT + file LICENSE
Encoding: UTF-8
biocViews: Spatial, SingleCell, Transcriptomics, GeneExpression,
        QualityControl, DimensionReduction
Depends: ggplot2
Imports: SpatialExperiment, SingleCellExperiment, SummarizedExperiment,
        ggside, grid, ggrepel, RColorBrewer, scales, grDevices,
        methods, stats
VignetteBuilder: knitr
Suggests: BiocStyle, rmarkdown, knitr, OSTA.data, VisiumIO, arrow,
        STexampleData, BumpyMatrix, scater, scran, uwot, testthat,
        patchwork
RoxygenNote: 7.3.3
git_url: https://git.bioconductor.org/packages/ggspavis
git_branch: devel
git_last_commit: 7872157
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:55:27 UTC; biocbuild
Author: Lukas M. Weber [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3282-1730>),
  Helena L. Crowell [aut] (ORCID:
    <https://orcid.org/0000-0002-4801-1767>),
  Yixing E. Dong [aut] (ORCID: <https://orcid.org/0009-0003-5115-5686>)
Maintainer: Lukas M. Weber <lmweb012@gmail.com>
