# Generated by roxygen2: do not edit by hand

export(plotCoords)
export(plotDimRed)
export(plotFeatureQC)
export(plotMolecules)
export(plotObsQC)
export(plotQC)
export(plotSpotQC)
export(plotSpots)
export(plotVisium)
importFrom(RColorBrewer,brewer.pal)
importFrom(SingleCellExperiment,'reducedDim<-')
importFrom(SingleCellExperiment,counts)
importFrom(SingleCellExperiment,reducedDim)
importFrom(SingleCellExperiment,reducedDimNames)
importFrom(SpatialExperiment,'imgData<-')
importFrom(SpatialExperiment,imgData)
importFrom(SpatialExperiment,imgRaster)
importFrom(SpatialExperiment,scaleFactors)
importFrom(SpatialExperiment,spatialCoords)
importFrom(SpatialExperiment,spatialCoordsNames)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(ggplot2,.data)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,geom_violin)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_colorbar)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,layer)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_color_gradient)
importFrom(ggplot2,scale_color_gradientn)
importFrom(ggplot2,scale_color_identity)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_color_viridis_c)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(ggplot2,scale_fill_identity)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_fill_viridis_c)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_reverse)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,unit)
importFrom(ggplot2,xlab)
importFrom(ggrepel,geom_text_repel)
importFrom(ggside,geom_xsidehistogram)
importFrom(ggside,geom_ysidehistogram)
importFrom(grDevices,colorRampPalette)
importFrom(grid,rasterGrob)
importFrom(methods,as)
importFrom(methods,is)
importFrom(scales,hue_pal)
importFrom(stats,median)
importFrom(stats,setNames)
